(* Copyright © 2026 Stéphane Glondu. Licensed under GPL-2+. *)

open Support
open Common

let main = function
  | cmd :: args -> (
      match Filename.basename cmd with
      | "dh_ocamllibinstall" -> Dh_ocamllibinstall.main args
      | _ -> exit 1)
  | _ -> exit 1

let () = getArgs () |> main
