(* Copyright © 2026 Stéphane Glondu. Licensed under GPL-2+. *)

val getArgs : unit -> string list
val exit : int -> 'a
val loadFile : string -> string option
val saveFile : string -> string -> unit
val pread : string -> string option
val printf : ('a, unit, string, unit) format4 -> 'a
