#!/bin/sh

set -e

BASE="${1%.bc.js}"
: ${NODE_PATH:=/usr/share/nodejs}
export NODE_PATH

{
    echo 'import { TextEncoder, TextDecoder } from "text-encoding";'
    cat "${BASE}.bc.js"
} > "${BASE}.in.js"

babeljs "${BASE}.in.js" -o "${BASE}.es.js"
esbuild "${BASE}.es.js" --minify --bundle --platform=neutral --target=es2023 --outfile="${BASE}.js"
